from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll33MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/6d5284b842434413a17133f7bf259669/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/6d5284b842434413a17133f7bf259669_roll_sideways/20240826-215603_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll34MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/6d5284b842434413a17133f7bf259669/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/6d5284b842434413a17133f7bf259669_roll_sideways/20240826-214013_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll35MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/6d5284b842434413a17133f7bf259669/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/6d5284b842434413a17133f7bf259669_roll_sideways/20240826-220927_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll36MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/6d5284b842434413a17133f7bf259669/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/6d5284b842434413a17133f7bf259669_roll_sideways/20240826-213503_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll37MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/6d5284b842434413a17133f7bf259669/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/6d5284b842434413a17133f7bf259669_roll_sideways/20240826-212113_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll38MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/6d5284b842434413a17133f7bf259669/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/6d5284b842434413a17133f7bf259669_roll_sideways/20240826-221022_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll294MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ea0cf48616e34708b73c82eb7c7366ca/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ea0cf48616e34708b73c82eb7c7366ca_roll_sideways/20240826-213625_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll295MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ea0cf48616e34708b73c82eb7c7366ca/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ea0cf48616e34708b73c82eb7c7366ca_roll_sideways/20240826-212657_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll296MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ea0cf48616e34708b73c82eb7c7366ca/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ea0cf48616e34708b73c82eb7c7366ca_roll_sideways/20240826-213512_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll297MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ea0cf48616e34708b73c82eb7c7366ca/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ea0cf48616e34708b73c82eb7c7366ca_roll_sideways/20240826-212634_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll298MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ea0cf48616e34708b73c82eb7c7366ca/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ea0cf48616e34708b73c82eb7c7366ca_roll_sideways/20240826-212040_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll299MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/ea0cf48616e34708b73c82eb7c7366ca/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/ea0cf48616e34708b73c82eb7c7366ca_roll_sideways/20240826-213445_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll873MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/7ca7ebcfad964498b49af73be442acf9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/7ca7ebcfad964498b49af73be442acf9_roll_sideways/20240826-211750_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll874MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/7ca7ebcfad964498b49af73be442acf9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/7ca7ebcfad964498b49af73be442acf9_roll_sideways/20240826-213339_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll875MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/7ca7ebcfad964498b49af73be442acf9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/7ca7ebcfad964498b49af73be442acf9_roll_sideways/20240826-215608_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll876MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/7ca7ebcfad964498b49af73be442acf9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/7ca7ebcfad964498b49af73be442acf9_roll_sideways/20240826-213308_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll877MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/7ca7ebcfad964498b49af73be442acf9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/7ca7ebcfad964498b49af73be442acf9_roll_sideways/20240826-213127_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotRollSidewaysToiletPaperRoll878MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="toilet paper roll",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/7ca7ebcfad964498b49af73be442acf9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/7ca7ebcfad964498b49af73be442acf9_roll_sideways/20240826-221404_no_interaction/trajectory-unified_wo_traj_v2.pkl"



from metasim.cfg.checkers import DetectedChecker, RelativeBboxDetector
from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass

from ...base_task_metacfg import BaseTaskMetaCfg


@configclass
class LiberoPickKetchupMetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.LIBERO
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 250

    objects = [
        RigidObjMetaCfg(
            name="bbq_sauce",
            physics=PhysicStateType.RIGIDBODY,
            filepath="roboverse_data/assets/libero/COMMON/stable_hope_objects/bbq_sauce/usd/bbq_sauce.usd",
            urdf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/bbq_sauce/urdf/bbq_sauce.urdf",
            mjcf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/bbq_sauce/mjcf/bbq_sauce.xml",
        ),
        RigidObjMetaCfg(
            name="basket",
            physics=PhysicStateType.RIGIDBODY,
            filepath="roboverse_data/assets/libero/COMMON/stable_hope_objects/basket/usd/basket.usd",
            urdf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/basket/urdf/basket.urdf",
            mjcf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/basket/mjcf/basket.xml",
        ),
        RigidObjMetaCfg(
            name="ketchup",
            physics=PhysicStateType.RIGIDBODY,
            filepath="roboverse_data/assets/libero/COMMON/stable_hope_objects/ketchup/usd/ketchup.usd",
            urdf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/ketchup/urdf/ketchup.urdf",
            mjcf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/ketchup/mjcf/ketchup.xml",
        ),
        RigidObjMetaCfg(
            name="salad_dressing",
            physics=PhysicStateType.RIGIDBODY,
            filepath="roboverse_data/assets/libero/COMMON/stable_hope_objects/salad_dressing/usd/salad_dressing.usd",
            urdf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/salad_dressing/urdf/salad_dressing.urdf",
            mjcf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/salad_dressing/mjcf/salad_dressing.xml",
        ),
        RigidObjMetaCfg(
            name="alphabet_soup",
            physics=PhysicStateType.RIGIDBODY,
            filepath="roboverse_data/assets/libero/COMMON/stable_hope_objects/alphabet_soup/usd/alphabet_soup.usd",
            urdf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/alphabet_soup/urdf/alphabet_soup.urdf",
            mjcf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/alphabet_soup/mjcf/alphabet_soup.xml",
        ),
        RigidObjMetaCfg(
            name="cream_cheese",
            physics=PhysicStateType.RIGIDBODY,
            filepath="roboverse_data/assets/libero/COMMON/stable_hope_objects/cream_cheese/usd/cream_cheese.usd",
            urdf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/cream_cheese/urdf/cream_cheese.urdf",
            mjcf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/cream_cheese/mjcf/cream_cheese.xml",
        ),
        RigidObjMetaCfg(
            name="milk",
            physics=PhysicStateType.RIGIDBODY,
            filepath="roboverse_data/assets/libero/COMMON/stable_hope_objects/milk/usd/milk.usd",
            urdf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/milk/urdf/milk.urdf",
            mjcf_path="roboverse_data/assets/libero/COMMON/stable_hope_objects/milk/mjcf/milk.xml",
        ),
    ]

    traj_filepath = "roboverse_data/trajs/libero/pick_up_the_ketchup_and_place_it_in_the_basket/v2/franka_v2.pkl"

    checker = DetectedChecker(
        obj_name="ketchup",
        obj_subpath=None,
        detector=RelativeBboxDetector(
            base_obj_name="basket",
            relative_pos=[0.0, 0.0, 0.07185],
            relative_quat=[1.0, 0.0, 0.0, 0.0],
            checker_lower=[-0.08, -0.08, -0.11],
            checker_upper=[0.08, 0.08, 0.05],
        ),
    )



import argparse
import time
from typing import Type

import gymnasium as gym
import torch
from loguru import logger as log
from omni.isaac.lab.app import AppLauncher

from metasim.cfg.objects import ArticulationObjMetaCfg, BaseObjMetaCfg, RigidObjMetaCfg
from metasim.cfg.robots import BaseRobotMetaCfg
from metasim.cfg.scenario import ScenarioMetaCfg
from metasim.types import Action, Dof, EnvState, Extra, Obs, Reward, Success, Termination, TimeOut

from ..base import BaseSimHandler, EnvWrapper, IdentityEnvWrapper
from .env_overwriter import IsaaclabEnvOverwriter
from .isaaclab_helper import get_pose, joint_is_implicit_actuator

try:
    from .empty_env import EmptyEnv
except:
    pass


class IsaaclabHandler(BaseSimHandler):
    def __init__(self, scenario: ScenarioMetaCfg, num_envs: int = 1, headless: bool = False):
        super().__init__(scenario, num_envs, headless)
        self.checker_debug_viewers = scenario.task.checker.get_debug_viewers()

    def launch(self) -> None:
        super().launch()
        TaskName = self.task.__class__.__name__.removesuffix("MetaCfg")
        env_overwriter = IsaaclabEnvOverwriter(self.scenario)
        gym.register(
            id=TaskName,
            entry_point="metasim.sim.isaaclab.empty_env:EmptyEnv",
            disable_env_checker=True,
            order_enforce=False,
            kwargs={
                "env_cfg_entry_point": "metasim.sim.isaaclab.empty_env:EmptyEnvCfg",
                "_setup_scene": env_overwriter._setup_scene,
                "_reset_idx": env_overwriter._reset_idx,
                "_pre_physics_step": env_overwriter._pre_physics_step,
                "_apply_action": env_overwriter._apply_action,
                "_get_observations": env_overwriter._get_observations,
                "_get_rewards": env_overwriter._get_rewards,
                "_get_dones": env_overwriter._get_dones,
            },
        )
        parser = argparse.ArgumentParser()
        AppLauncher.add_app_launcher_args(parser)
        args = parser.parse_args([])
        args.enable_cameras = True
        args.headless = self.headless

        ## Only set args.renderer seems not enough
        if self.scenario.render.mode == "raytracing":
            args.renderer = "RayTracedLighting"
        elif self.scenario.render.mode == "pathtracing":
            args.renderer = "PathTracing"
        elif self.scenario.render.mode == "rasterization":
            raise ValueError("Isaaclab does not support rasterization")
        else:
            raise ValueError(f"Unknown render mode: {self.scenario.render.mode}")

        app_launcher = AppLauncher(args)
        self.simulation_app = app_launcher.app

        from omni.isaac.lab_tasks.utils import parse_env_cfg

        env_cfg = parse_env_cfg(TaskName)
        env_cfg.scene.num_envs = self.num_envs
        env_cfg.decimation = self.task.decimation
        env_cfg.episode_length_s = self.task.episode_length * (1 / 60) * self.task.decimation

        self.env: EmptyEnv = gym.make(TaskName, cfg=env_cfg)

        ## Render mode setting, must be done after isaaclab is launched
        ## For more info, see the import below
        import carb
        import omni.replicator.core as rep

        # from omni.rtx.settings.core.widgets.pt_widgets import PathTracingSettingsFrame

        rep.settings.set_render_rtx_realtime()  # fix noising rendered images

        settings = carb.settings.get_settings()
        if self.scenario.render.mode == "pathtracing":
            settings.set_string("/rtx/rendermode", "PathTracing")
        elif self.scenario.render.mode == "raytracing":
            settings.set_string("/rtx/rendermode", "RayTracedLighting")
        elif self.scenario.render.mode == "rasterization":
            raise ValueError("Isaaclab does not support rasterization")
        else:
            raise ValueError(f"Unknown render mode: {self.scenario.render.mode}")

        log.info(f"Render mode: {settings.get_as_string('/rtx/rendermode')}")
        log.info(f"Render totalSpp: {settings.get('/rtx/pathtracing/totalSpp')}")
        log.info(f"Render spp: {settings.get('/rtx/pathtracing/spp')}")
        log.info(f"Render adaptiveSampling/enabled: {settings.get('/rtx/pathtracing/adaptiveSampling/enabled')}")
        log.info(f"Render maxBounces: {settings.get('/rtx/pathtracing/maxBounces')}")

    ############################################################
    ## Gymnasium main methods
    ############################################################
    def step(self, action: list[Action]) -> tuple[Obs, Reward, Success, TimeOut, Extra]:
        joint_names = self.get_object_joint_names(self.robot)
        action_env_tensors = torch.zeros((self.num_envs, len(joint_names)), device=self.env.device)
        for env_id in range(self.num_envs):
            action_env = action[env_id]
            for i, joint_name in enumerate(joint_names):
                # Convert numpy float32 to torch tensor and move to correct device
                action_env_tensors[env_id, i] = torch.tensor(
                    action_env["dof_pos_target"][joint_name], device=self.env.device
                )

        _, _, _, _, extras = self.env.step(action_env_tensors)
        time_out = self.get_time_out()
        success = self.get_success()
        obs = self.get_observation()
        reward = self.get_reward()

        return obs, reward, success, time_out, extras

    def reset(self, env_ids: list[int] | None = None) -> tuple[Obs, Extra]:
        if env_ids is None:
            env_ids = list(range(self.num_envs))

        log.info(f"Resetting envs {env_ids}")

        tic = time.time()
        _, extras = self.env.reset(env_ids=env_ids)
        toc = time.time()
        log.trace(f"Reset isaaclab env time: {toc - tic:.2f}s")

        tic = time.time()
        self.task.checker.reset(self, env_ids=env_ids)
        toc = time.time()
        log.trace(f"Reset checker time: {toc - tic:.2f}s")

        ## Force rerender, see https://isaac-sim.github.io/IsaacLab/main/source/refs/issues.html#blank-initial-frames-from-the-camera
        tic = time.time()
        # XXX: previously 12 is not enough for pick_cube, if this is the case again, try 18
        for _ in range(12):
            # XXX: previously sim.render() is not enough for pick_cube, if this is the case again, try calling sim.step()
            # self.env.sim.step()
            self.env.sim.render()
        toc = time.time()
        log.trace(f"Reset render time: {toc - tic:.2f}s")

        ## Update camera buffer
        tic = time.time()
        for sensor in self.env.scene.sensors.values():
            sensor.update(dt=0)
        toc = time.time()
        log.trace(f"Reset sensor buffer time: {toc - tic:.2f}s")

        ## Update obs
        tic = time.time()
        obs = self.get_observation()
        toc = time.time()
        log.trace(f"Reset getting obs time: {toc - tic:.2f}s")

        return obs, extras

    def render(self) -> None:
        pass

    def close(self) -> None:
        super().close()
        self.env.close()
        self.simulation_app.close()

    ############################################################
    ## Utils
    ############################################################
    def refresh_render(self) -> None:
        for sensor in self.env.scene.sensors.values():
            sensor.update(dt=0)
        self.env.sim.render()

    def get_observation(self) -> Obs:
        obs = self.env._get_observations()
        # obs["states"] = self.get_states()  # TODO: This will increase 50% more step time, find a better way
        return obs

    def get_reward(self) -> Reward:
        from metasim.cfg.tasks.base_task_metacfg import BaseRLTaskMetaCfg

        if hasattr(self.task, "reward_fn"):
            return self.task.reward_fn(self.get_states())

        elif isinstance(self.task, BaseRLTaskMetaCfg):
            final_reward = torch.zeros(self.num_envs)
            for reward_func, reward_weight in zip(self.task.reward_functions, self.task.reward_weights):
                final_reward += reward_func(self.get_states()) * reward_weight
            return final_reward
        else:
            return None

    def get_success(self) -> Success:
        return self.task.checker.check(self)

    def get_time_out(self) -> TimeOut:
        _, time_out = self.env._get_dones()
        time_out = time_out.cpu()
        return time_out

    def get_termination(self) -> Termination:
        ## TODO: implement termination better
        # log.error("get_termination() not implemented, please override this method")
        return None

    ############################################################
    ## Set states
    ############################################################
    def _set_object_pose(
        self,
        object: BaseObjMetaCfg,
        position: torch.Tensor,  # (num_envs, 3)
        rotation: torch.Tensor,  # (num_envs, 4)
        env_ids: list[int] | None = None,
    ) -> None:
        """
        Set the pose of an object, set the velocity to zero
        """
        if env_ids is None:
            env_ids = list(range(self.num_envs))

        assert position.shape == (len(env_ids), 3)
        assert rotation.shape == (len(env_ids), 4)

        if isinstance(object, ArticulationObjMetaCfg):
            obj_inst = self.env.scene.articulations[object.name]
        elif isinstance(object, RigidObjMetaCfg):
            obj_inst = self.env.scene.rigid_objects[object.name]
        else:
            raise ValueError(f"Invalid object type: {type(object)}")

        pose = torch.concat(
            [
                position.to(self.env.device, dtype=torch.float32) + self.env.scene.env_origins[env_ids],
                rotation.to(self.env.device, dtype=torch.float32),
            ],
            dim=-1,
        )
        obj_inst.write_root_pose_to_sim(pose, env_ids=torch.tensor(env_ids, device=self.env.device))
        obj_inst.write_root_velocity_to_sim(
            torch.zeros((len(env_ids), 6), device=self.env.device, dtype=torch.float32),
            env_ids=torch.tensor(env_ids, device=self.env.device),
        )  # ! critical
        obj_inst.write_data_to_sim()

    def _set_object_joint_pos(
        self,
        object: BaseObjMetaCfg,
        joint_pos: torch.Tensor,  # (num_envs, num_joints)
        env_ids: list[int] | None = None,
    ) -> None:
        if env_ids is None:
            env_ids = list(range(self.num_envs))
        assert joint_pos.shape[0] == len(env_ids)
        pos = joint_pos.to(self.env.device)
        vel = torch.zeros_like(pos)
        obj_inst = self.env.scene.articulations[object.name]
        obj_inst.write_joint_state_to_sim(pos, vel, env_ids=torch.tensor(env_ids, device=self.env.device))
        obj_inst.write_data_to_sim()

    def set_states(self, states: list[EnvState], env_ids: list[int] | None = None) -> None:
        if env_ids is None:
            env_ids = list(range(self.num_envs))
        for obj in self.objects + [self.robot]:
            if obj.name not in states[0]:
                log.warning(f"Missing {obj.name} in states, setting its velocity to zero")
                pos, rot = get_pose(self.env, obj.name, env_ids=env_ids)
                self._set_object_pose(obj, pos, rot, env_ids=env_ids)
                continue

            if states[0][obj.name].get("pos", None) is None or states[0][obj.name].get("rot", None) is None:
                log.warning(f"No pose found for {obj.name}, setting its velocity to zero")
                pos, rot = get_pose(self.env, obj.name, env_ids=env_ids)
                self._set_object_pose(obj, pos, rot, env_ids=env_ids)
            else:
                pos = torch.stack([states[env_id][obj.name]["pos"] for env_id in env_ids]).to(self.env.device)
                rot = torch.stack([states[env_id][obj.name]["rot"] for env_id in env_ids]).to(self.env.device)
                self._set_object_pose(obj, pos, rot, env_ids=env_ids)

            if isinstance(obj, ArticulationObjMetaCfg):
                if states[0][obj.name].get("dof_pos", None) is None:
                    log.warning(f"No dof_pos found for {obj.name}")
                else:
                    dof_dict = [states[env_id][obj.name]["dof_pos"] for env_id in env_ids]
                    joint_names = self.get_object_joint_names(obj)
                    joint_pos = torch.zeros((len(env_ids), len(joint_names)), device=self.env.device)
                    for i, joint_name in enumerate(joint_names):
                        joint_pos[:, i] = torch.Tensor([x[joint_name] for x in dof_dict])
                    self._set_object_joint_pos(obj, joint_pos, env_ids=env_ids)
                    if obj == self.robot:
                        robot_inst = self.env.scene.articulations[obj.name]
                        robot_inst.set_joint_position_target(
                            joint_pos, env_ids=torch.tensor(env_ids, device=self.env.device)
                        )
                        robot_inst.write_data_to_sim()

    def set_pose(self, obj_name: str, pos: torch.Tensor, rot: torch.Tensor) -> None:
        # XXX: This is so hacky
        obj = next(
            (obj for obj in self.objects + [self.robot] + self.checker_debug_viewers if obj.name == obj_name), None
        )
        self._set_object_pose(obj, pos, rot)

    ############################################################
    ## Get states
    ############################################################
    def get_states(self, env_ids: list[int] | None = None) -> list[EnvState]:
        if env_ids is None:
            env_ids = list(range(self.num_envs))
        states = []
        for env_id in env_ids:
            env_state = {}
            for obj in self.objects + [self.robot]:
                if isinstance(obj, ArticulationObjMetaCfg):
                    obj_inst = self.env.scene.articulations[obj.name]
                else:
                    obj_inst = self.env.scene.rigid_objects[obj.name]
                obj_state = {}

                ## Basic states
                obj_state["pos"] = obj_inst.data.root_pos_w[env_id].cpu() - self.env.scene.env_origins[env_id].cpu()
                obj_state["rot"] = obj_inst.data.root_quat_w[env_id].cpu()
                obj_state["vel"] = obj_inst.data.root_vel_w[env_id].cpu()
                obj_state["ang_vel"] = obj_inst.data.root_ang_vel_w[env_id].cpu()

                ## Joint states
                if isinstance(obj, ArticulationObjMetaCfg):
                    obj_state["dof_pos"] = {
                        joint_name: obj_inst.data.joint_pos[env_id][i].item()
                        for i, joint_name in enumerate(obj_inst.joint_names)
                    }
                    obj_state["dof_vel"] = {
                        joint_name: obj_inst.data.joint_vel[env_id][i].item()
                        for i, joint_name in enumerate(obj_inst.joint_names)
                    }

                ## Actuator states
                ## XXX: Could non-robot objects have actuators?
                if isinstance(obj, BaseRobotMetaCfg):
                    obj_state["dof_pos_target"] = {
                        joint_name: obj_inst.data.joint_pos_target[env_id][i].item()
                        for i, joint_name in enumerate(obj_inst.joint_names)
                    }
                    obj_state["dof_vel_target"] = {
                        joint_name: obj_inst.data.joint_vel_target[env_id][i].item()
                        for i, joint_name in enumerate(obj_inst.joint_names)
                    }
                    obj_state["dof_torque"] = {
                        joint_name: (
                            obj_inst.data.joint_effort_target[env_id][i].item()
                            if joint_is_implicit_actuator(joint_name, obj_inst)
                            else obj_inst.data.applied_torque[env_id][i].item()
                        )
                        for i, joint_name in enumerate(obj_inst.joint_names)
                    }
                env_state[obj.name] = obj_state

                ## Body states
                ### XXX: will have bug when there are multiple objects with same structure, e.g. dual gripper
                coms = obj_inst.root_physx_view.get_coms()
                coms = coms.reshape((self.env.num_envs, obj_inst.num_bodies, 7))
                com_positions = coms[:, :, :3]  # (num_envs, num_bodies, 3)
                for i, body_name in enumerate(obj_inst.body_names):
                    body_state = {}
                    body_state["pos"] = (
                        obj_inst.data.body_pos_w[env_id, i].cpu() - self.env.scene.env_origins[env_id].cpu()
                    )
                    body_state["rot"] = obj_inst.data.body_quat_w[env_id, i].cpu()
                    body_state["vel"] = obj_inst.data.body_lin_vel_w[env_id, i].cpu()
                    body_state["ang_vel"] = obj_inst.data.body_ang_vel_w[env_id, i].cpu()
                    body_state["com"] = com_positions[env_id, i].cpu()
                    env_state[f"metasim_body_{body_name}"] = body_state

            states.append(env_state)
        return states

    def get_pos(
        self,
        obj_name: str,
        env_ids: list[int] | None = None,
        obj_subpath: str | None = None,
    ) -> torch.FloatTensor:
        if env_ids is None:
            env_ids = list(range(self.num_envs))
        pos, _ = get_pose(self.env, obj_name, obj_subpath, env_ids=env_ids)
        assert pos.shape == (len(env_ids), 3)
        return pos

    def get_rot(
        self,
        obj_name: str,
        env_ids: list[int] | None = None,
        obj_subpath: str | None = None,
    ) -> torch.FloatTensor:
        if env_ids is None:
            env_ids = list(range(self.num_envs))
        _, rot = get_pose(self.env, obj_name, obj_subpath, env_ids=env_ids)
        assert rot.shape == (len(env_ids), 4)
        return rot

    def get_dof_pos(self, obj_name: str, joint_name: str, env_ids: list[int] | None = None) -> torch.FloatTensor:
        if env_ids is None:
            env_ids = list(range(self.num_envs))
        dof_pos = torch.zeros(len(env_ids))
        for i, env_id in enumerate(env_ids):
            dof_pos[i] = self.env.scene.articulations[obj_name].data.joint_pos[env_id][
                self.env.scene.articulations[obj_name].joint_names.index(joint_name)
            ]
        return dof_pos

    ############################################################
    ## Misc
    ############################################################
    def is_running(self) -> bool:
        return self.simulation_app.is_running()

    def get_object_joint_names(self, object: BaseObjMetaCfg) -> list[str]:
        assert isinstance(object, ArticulationObjMetaCfg)
        return self.env.scene.articulations[object.name].joint_names

    def get_robot_joint_limits(self) -> tuple[Dof, Dof]:
        dof_min_tensor, dof_max_tensor = self.env.robot.root_physx_view.get_dof_limits()[0].split(1, dim=-1)
        dof_min = {
            joint_name: dof_min_tensor[i].item() for i, joint_name in enumerate(self.get_object_joint_names(self.robot))
        }
        dof_max = {
            joint_name: dof_max_tensor[i].item() for i, joint_name in enumerate(self.get_object_joint_names(self.robot))
        }
        return dof_min, dof_max

    @property
    def episode_length_buf(self) -> list[int]:
        return self.env.episode_length_buf.tolist()


IsaaclabEnv: Type[EnvWrapper[IsaaclabHandler]] = IdentityEnvWrapper(IsaaclabHandler)



from __future__ import annotations

import torch

from metasim.cfg.checkers import PushChecker
from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.types import EnvState
from metasim.utils import configclass
from metasim.utils.h1_utils import left_hand_position

from .base_locomotion_env import HumanoidTaskMetaCfg


class SuccessReward:
    def __init__(self, robot_name="h1_simple_hand"):
        self._robot_name = robot_name
        if robot_name != "h1" and robot_name != "h1_simple_hand":
            raise ValueError(f"Unknown robot {robot_name} without hand")

    def __call__(self, states: list[EnvState]) -> float:
        state = states[0]
        box_pos = state["object"]["pos"]
        dest_pos = state["destination"]["pos"]
        dgoal = torch.norm(box_pos - dest_pos)
        return float(dgoal < 0.05)


class GoalDistanceReward:
    def __init__(self, robot_name="h1_simple_hand"):
        self._robot_name = robot_name

    def __call__(self, states: list[EnvState]) -> torch.FloatTensor:
        state = states[0]
        box_pos = state["object"]["pos"]
        dest_pos = state["destination"]["pos"]
        return -torch.norm(box_pos - dest_pos)


class HandDistanceReward:
    def __init__(self, robot_name="h1_simple_hand"):
        self._robot_name = robot_name
        if robot_name != "h1" and robot_name != "h1_simple_hand":
            raise ValueError(f"Unknown robot {robot_name} without hand")

    def __call__(self, states: list[EnvState]) -> torch.FloatTensor:
        state = states[0]
        box_pos = state["object"]["pos"]
        left_hand_pos = left_hand_position(state, self._robot_name)
        return -torch.norm(box_pos - left_hand_pos)


@configclass
class PushMetaCfg(HumanoidTaskMetaCfg):
    episode_length = 500  # Updated to match the specification
    objects = [
        RigidObjMetaCfg(
            name="table",
            mjcf_path="roboverse_data/assets/humanoidbench/push/table/mjcf/table.xml",
            physics=PhysicStateType.GEOM,
            fix_base_link=True,
        ),
        RigidObjMetaCfg(
            name="object",
            mjcf_path="roboverse_data/assets/humanoidbench/push/object/mjcf/object.xml",
            physics=PhysicStateType.GEOM,
        ),
        RigidObjMetaCfg(
            name="destination",
            mjcf_path="roboverse_data/assets/humanoidbench/push/destination/mjcf/destination.xml",
            physics=PhysicStateType.GEOM,
            fix_base_link=True,
        ),
    ]
    traj_filepath = "roboverse_data/trajs/humanoidbench/push/v2/initial_state_v2.json"
    checker = PushChecker()
    reward_weights = [1000, 1, 0.1]  # αs, αt, αh
    reward_functions = [SuccessReward, GoalDistanceReward, HandDistanceReward]



import os

# Template for the Python file
template = """

@configclass
class {class_name}MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        {objects}
    ]
    traj_filepath = "{traj_filepath}"
"""


# Generate object definitions
def generate_object_def(obj):
    return f"""RigidObjMetaCfg(
            name="{obj["name"]}",
            urdf_path="{obj["urdf_path"]}",
            physics=PhysicStateType.RIGIDBODY
        )"""


import glob
import json
import pickle


def dict_to_pkl(data, filename):
    with open(filename, "wb") as f:
        pickle.dump(data, f)


# CAT = "task_refine_rot_only"
# CAT_TAG = "Rot"


# CAT = "task_refine_pos"
# CAT_TAG = "Pos"


CAT = "task_refine_6dof"
CAT_TAG = "PosRot"
ROOT = f"/home/haoran/Project/RoboVerse/RoboVerse/Open6DOR/Open6DOR/assets/tasks/{CAT}"
tasks = glob.glob(f"{ROOT}/*/*/*/task_config_new5.json")


def to_camel_case(s):
    # Split the string by underscores and capitalize each part
    parts = s.split("_")
    # Keep the first part unchanged and capitalize the rest
    return parts[0].capitalize() + "".join(word.capitalize() for word in parts[1:])


current = 0
total = len(tasks)
for task_i, task in enumerate(tasks):
    current += 1
    print(f"{current}/{total}")
    with open(task) as f:
        config = json.load(f)
    obj_names = config["selected_obj_names"]
    obj_urdfs = config["selected_urdfs"]
    instruction = config["instruction"]
    init_obj_pos = config["init_obj_pos"]
    if CAT_TAG == "Rot":
        CateTag = to_camel_case(config["rotation_instruction_label"].replace(" ", "_"))
        ObjTag = to_camel_case(config["target_obj_name"].replace(" ", "_"))
    elif CAT_TAG == "Pos":
        CateTag = to_camel_case(config["position_tag"].replace(" ", "_"))
        ObjTag = to_camel_case(config["target_obj_name"].replace(" ", "_"))
    elif CAT_TAG == "PosRot":
        PosTag = to_camel_case(config["position_tag"].replace(" ", "_"))
        CateTag = to_camel_case(config["position_tag"].replace(" ", "_")) + to_camel_case(
            config["rotation_instruction_label"].replace(" ", "_")
        )
        ObjTag = to_camel_case(config["target_obj_name"].replace(" ", "_"))
    else:
        raise ValueError(f"Invalid CAT_TAG: {CAT_TAG}")
    # Generate Python files
    output_dir = f"metasim/cfg/tasks/open6dor/task/{CAT_TAG}"
    os.makedirs(output_dir, exist_ok=True)
    # import pdb; pdb.set_trace()
    class_name = f"Opensdor{CAT_TAG}{CateTag}{ObjTag}{task_i}"
    cfg = {
        "file_name": f"{PosTag}.py",
        "objects": [
            {
                "name": obj_names[i],
                "urdf_path": "data_isaaclab/assets/open6dor/" + obj_urdfs[i],
            }
            for i in range(len(obj_names))
        ],
    }
    tag4 = task.split("/")[-4]
    tag3 = task.split("/")[-3]
    tag2 = task.split("/")[-2]
    root_new = f"data_isaaclab/source_data/open6dor/{CAT}/{tag4}/{tag3}/{tag2}"
    traj_filepath = f"{root_new}/trajectory-unified_wo_traj_v2.pkl"
    # import pdb; pdb.set_trace()
    # for cfg in config_list:
    object_defs = ",\n        ".join([generate_object_def(obj) for obj in cfg["objects"]])

    file_content = template.format(
        class_name=class_name,
        objects=object_defs,
        traj_filepath=traj_filepath,
    )
    if not os.path.exists(os.path.join(output_dir, cfg["file_name"])):
        with open(os.path.join(output_dir, cfg["file_name"]), "a") as file:
            file_prefix = """import math

from metasim.cfg.checkers import DetectedChecker, OrOp, RelativeBboxDetector
from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass
"""
            file.write(file_prefix)

    with open(os.path.join(output_dir, cfg["file_name"]), "a") as file:
        file.write(file_content)

# print(f"Generated {len(config_list)} configuration files in '{output_dir}' directory.")



from metasim.cfg.objects import ArticulationObjMetaCfg, RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class GetIceFromFridgeMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 600
    traj_filepath = "roboverse_data/trajs/rlbench/get_ice_from_fridge/v2"
    objects = [
        ArticulationObjMetaCfg(
            name="fridge_base",
            filepath="roboverse_data/assets/rlbench/get_ice_from_fridge/fridge_base/usd/fridge_base.usd",
        ),
        RigidObjMetaCfg(
            name="cup_visual",
            filepath="roboverse_data/assets/rlbench/get_ice_from_fridge/cup_visual/usd/cup_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    # TODO: add checker



from metasim.cfg.checkers import DetectedChecker, RelativeBboxDetector
from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class BasketballInHoopMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/basketball_in_hoop/v2"
    objects = [
        RigidObjMetaCfg(
            name="basket_ball_hoop_visual",
            filepath="roboverse_data/assets/rlbench/basketball_in_hoop/basket_ball_hoop_visual/usd/basket_ball_hoop_visual_colored.usd",
            mesh_path="roboverse_data/assets/rlbench/basketball_in_hoop/basket_ball_hoop_visual/mesh/basket_ball_hoop_visual_colored.obj",
            physics=PhysicStateType.XFORM,
        ),
        RigidObjMetaCfg(
            name="ball",
            filepath="roboverse_data/assets/rlbench/basketball_in_hoop/ball/usd/ball_textured.usd",
            mesh_path="roboverse_data/assets/rlbench/basketball_in_hoop/ball/mesh/ball_textured.obj",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    checker = DetectedChecker(
        obj_name="ball",
        detector=RelativeBboxDetector(
            base_obj_name="basket_ball_hoop_visual",
            relative_quat=[-0.52125348147482, 0.66573167191783, 0.4157939106384, 0.33497995900038],
            relative_pos=[-0.060456029040702, -0.27864555866622, -0.17772846479924],
            checker_lower=[-0.025, -0.025, -0.05],
            checker_upper=[0.025, 0.025, 0.1],
            # debug_vis=True,
        ),
    )



from metasim.cfg.objects import PrimitiveCubeMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass

from ..base_task_metacfg import BaseTaskMetaCfg


@configclass
class StackD0MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.ROBOSUITE
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 200
    traj_filepath = "data_isaaclab/source_data/robosuite/stack_d0/stack_d0_v2.pkl"
    objects = [
        PrimitiveCubeMetaCfg(
            name="cubeA", size=[0.02, 0.02, 0.02], mass=0.02, physics=PhysicStateType.RIGIDBODY, color=[1.0, 0.0, 0.0]
        ),
        PrimitiveCubeMetaCfg(
            name="cubeB",
            size=[0.025, 0.025, 0.025],
            mass=0.02,
            physics=PhysicStateType.RIGIDBODY,
            color=[0.0, 1.0, 0.0],
        ),
        PrimitiveCubeMetaCfg(
            name="table",
            # texture = 'data_isaaclab/robosuite/textures/ceramic.png',
            size=[0.8, 0.8, 0.05],
            physics=PhysicStateType.GEOM,
            color=[1.0, 1.0, 1.0],
        ),
    ]



from metasim.cfg.objects import ArticulationObjMetaCfg, PrimitiveCubeMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class PutItemInDrawerMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/put_item_in_drawer/v2"
    objects = [
        ArticulationObjMetaCfg(
            name="drawer_frame",
            filepath="roboverse_data/assets/rlbench/put_item_in_drawer/drawer_frame/usd/drawer_frame.usd",
        ),
        PrimitiveCubeMetaCfg(
            name="item",
            size=[0.04, 0.04, 0.04],
            color=[0.85, 0.85, 1.0],
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    # TODO: add checker



from __future__ import annotations

from dataclasses import MISSING

import torch

from metasim.cfg.objects import BaseObjMetaCfg
from metasim.utils.configclass import configclass

from .base_checker import BaseChecker

try:
    from metasim.sim.base import BaseSimHandler
except:
    pass


@configclass
class AndOp(BaseChecker):
    checkers: list[BaseChecker] = MISSING

    def reset(self, handler: BaseSimHandler, env_ids: list[int] | None = None):
        for checker in self.checkers:
            checker.reset(handler, env_ids=env_ids)

    def check(self, handler: BaseSimHandler) -> torch.BoolTensor:
        success = torch.ones(handler.num_envs, dtype=torch.bool)
        for checker in self.checkers:
            success = success & checker.check(handler)
        return success

    def get_debug_viewers(self) -> list[BaseObjMetaCfg]:
        viewers = []
        for checker in self.checkers:
            viewers += checker.get_debug_viewers()
        return viewers


@configclass
class OrOp(BaseChecker):
    checkers: list[BaseChecker] = MISSING

    def reset(self, handler: BaseSimHandler, env_ids: list[int] | None = None):
        for checker in self.checkers:
            checker.reset(handler, env_ids=env_ids)

    def check(self, handler: BaseSimHandler) -> torch.BoolTensor:
        success = torch.zeros(handler.num_envs, dtype=torch.bool)
        for checker in self.checkers:
            success = success | checker.check(handler)
        return success

    def get_debug_viewers(self) -> list[BaseObjMetaCfg]:
        viewers = []
        for checker in self.checkers:
            viewers += checker.get_debug_viewers()
        return viewers


@configclass
class NotOp(BaseChecker):
    checker: BaseChecker = MISSING

    def reset(self, handler: BaseSimHandler, env_ids: list[int] | None = None):
        self.checker.reset(handler, env_ids=env_ids)

    def check(self, handler: BaseSimHandler) -> torch.BoolTensor:
        return ~self.checker.check(handler)

    def get_debug_viewers(self) -> list[BaseObjMetaCfg]:
        return self.checker.get_debug_viewers()



from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class BeatTheBuzzMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/beat_the_buzz/v2"
    objects = [
        RigidObjMetaCfg(
            name="wand",
            filepath="roboverse_data/assets/rlbench/beat_the_buzz/wand/usd/wand.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
        RigidObjMetaCfg(
            name="Cuboid",
            filepath="roboverse_data/assets/rlbench/beat_the_buzz/Cuboid/usd/Cuboid.usd",
            physics=PhysicStateType.GEOM,
        ),
    ]
    # TODO: add checker



# ruff: noqa: F401

import time

from loguru import logger as log

from .base_task_metacfg import BaseTaskMetaCfg


def __get_quick_ref():
    tic = time.time()

    from .assembly_metacfg import AssemblyMetaCfg
    from .bin_pick_metacfg import BinPickMetaCfg
    from .calvin.calvin import MoveSliderLeftAMetaCfg
    from .debug.reach_origin_metacfg import ReachOriginMetaCfg
    from .dmcontrol.walker_walk_metacfg import WalkerWalkMetaCfg
    from .drawer_open_metacfg import DrawerOpenMetaCfg
    from .fetch import FetchCloseBoxMetaCfg
    from .gapartnet import GapartnetOpenDrawerMetaCfg
    from .hammer_metacfg import HammerMetaCfg
    from .humanoidbench import (
        BalanceMetaCfg,
        CrawlMetaCfg,
        HurdleMetaCfg,
        MazeMetaCfg,
        PoleMetaCfg,
        PushMetaCfg,
        RunMetaCfg,
        SitMetaCfg,
        SlideMetaCfg,
        StairMetaCfg,
        StandMetaCfg,
        WalkMetaCfg,
    )
    from .isaacgym_envs.ant_isaacgym_metacfg import AntIsaacGymMetaCfg
    from .libero.libero_objects.libero_pick_alphabet_soup import LiberoPickAlphabetSoupMetaCfg
    from .libero.libero_objects.libero_pick_bbq_sauce import LiberoPickBbqSauceMetaCfg
    from .libero.libero_objects.libero_pick_butter import LiberoPickButterMetaCfg
    from .libero.libero_objects.libero_pick_chocolate_pudding import LiberoPickChocolatePuddingMetaCfg
    from .libero.libero_objects.libero_pick_cream_cheese import LiberoPickCreamCheeseMetaCfg
    from .libero.libero_objects.libero_pick_ketchup import LiberoPickKetchupMetaCfg
    from .libero.libero_objects.libero_pick_milk import LiberoPickMilkMetaCfg
    from .libero.libero_objects.libero_pick_orange_juice import LiberoPickOrangeJuiceMetaCfg
    from .libero.libero_objects.libero_pick_salad_dressing import LiberoPickSaladDressingMetaCfg
    from .libero.libero_objects.libero_pick_tomato_sauce import LiberoPickTomatoSauceMetaCfg
    from .maniskill.pick_cube_metacfg import PickCubeMetaCfg
    from .maniskill.pick_single_ycb import PickSingleYcbCrackerBoxMetaCfg
    from .maniskill.stack_cube_metacfg import StackCubeMetaCfg
    from .rlafford.rl_afford_open_door_metacfg import RLAffordOpenDoorMetaCfg
    from .rlbench.basketball_in_hoop_metacfg import BasketballInHoopMetaCfg
    from .rlbench.close_box_metacfg import CloseBoxMetaCfg
    from .robosuite import SquareD0MetaCfg, SquareD1MetaCfg, SquareD2MetaCfg, StackD0MetaCfg
    from .simpler_env.simpler_env_grasp_opened_coke_can_metacfg import SimplerEnvGraspOpenedCokeCanMetaCfg
    from .skillblender import G1BaseTaskMetaCfg, H1BaseTaskMetaCfg
    from .uh1 import MabaoguoMetaCfg

    toc = time.time()

    log.trace(f"Time taken to load quick ref: {toc - tic:.2f} seconds")

    return locals()


__quick_ref = __get_quick_ref()


def __getattr__(name):
    if name in __quick_ref:
        return __quick_ref[name]
    else:
        raise AttributeError(f"Module {__name__} has no attribute {name}")



from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.utils import configclass


@configclass
class H1BaseTaskMetaCfg(BaseTaskMetaCfg):
    episode_length = 250
    objects = []
    traj_filepath = "/home/haoran/Project/RoboVerse/RoboVerse/metasim/cfg/tasks/skillblender/h1/h1_example_v2.json"



from __future__ import annotations

import argparse
import time
from collections import deque

import numpy as np
import rootutils
from loguru import logger as log
from rich.logging import RichHandler
from stable_baselines3 import PPO
from stable_baselines3.common.callbacks import BaseCallback
from stable_baselines3.common.env_util import make_vec_env

import wandb
from wandb.integration.sb3 import WandbCallback

rootutils.setup_root(__file__, pythonpath=True)
log.configure(handlers=[{"sink": RichHandler(), "format": "{message}"}])

from metasim.cfg.scenario import ScenarioMetaCfg
from metasim.constants import SimType
from metasim.scripts.RL.wrapper_sb3 import MetaSimGymWrapper
from metasim.utils.setup_util import get_robot, get_task


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--num_envs", type=int, default=1)
    parser.add_argument("--sim", type=str, default="isaaclab", choices=["isaaclab", "isaacgym", "mujoco"])
    parser.add_argument(
        "--robot",
        type=str,
        default="h1",
        choices=[
            "franka",
            "ur5e_2f85",
            "sawyer",
            "franka_with_gripper_extension",
            "h1_2_without_hand",
            "h1",
            "h1_simple_hand",
        ],
    )
    parser.add_argument("--task", type=str, default="Stand")
    parser.add_argument("--add_table", action="store_true")
    parser.add_argument("--total_timesteps", type=int, default=20000000)
    parser.add_argument("--train_or_eval", type=str, default="train", choices=["train", "eval", "test"])
    parser.add_argument("--model_path", type=str, default=None)
    # PPO specific arguments
    parser.add_argument("--learning_rate", type=float, default=3e-5)
    parser.add_argument("--n_steps", type=int, default=2048)
    parser.add_argument("--batch_size", type=int, default=512)
    parser.add_argument("--n_epochs", type=int, default=10)
    # Wandb arguments
    parser.add_argument("--use_wandb", action="store_true", default=True)
    parser.add_argument("--wandb_project", type=str, default="metasim_rl_training")
    parser.add_argument("--wandb_entity", type=str, default="<your_wandb_entity>")
    args = parser.parse_args()
    return args


class EpisodeLogCallback(BaseCallback):
    """
    Custom callback for plotting additional values in tensorboard.
    """

    def __init__(self, verbose=0):
        super().__init__(verbose)
        self.returns_info = {
            "results/return": [],
            "results/episode_length": [],
            "results/success": [],
            "results/success_subtasks": [],
        }

    def _on_step(self) -> bool:
        infos = self.locals["infos"]
        for idx in range(len(infos)):
            curr_info = infos[idx]
            if "episode" in curr_info:
                self.returns_info["results/return"].append(curr_info["episode"]["r"])
                self.returns_info["results/episode_length"].append(curr_info["episode"]["l"])
                cur_info_success = curr_info.get("success", 0)
                self.returns_info["results/success"].append(cur_info_success)
                cur_info_success_subtasks = curr_info.get("success_subtasks", 0)
                self.returns_info["results/success_subtasks"].append(cur_info_success_subtasks)
        return True

    def _on_rollout_end(self) -> None:
        # Use the model's num_timesteps (cumulative across envs) as the global step.
        global_step = self.model.num_timesteps
        for key in self.returns_info.keys():
            if self.returns_info[key]:
                self.logger.record(key, np.mean(self.returns_info[key]), global_step)
                self.returns_info[key] = []


class EvalCallback(BaseCallback):
    def __init__(self, eval_every: int = 100000, scenario=None, sim_type=None, verbose: int = 0):
        super().__init__(verbose=verbose)
        self.eval_every = eval_every
        self.eval_env = None
        self.scenario = scenario
        self.sim_type = sim_type

    def _on_training_start(self) -> None:
        self.eval_env = MetaSimGymWrapper(scenario=self.scenario, sim_type=self.sim_type, num_envs=1)

    def _on_step(self) -> bool:
        if self.num_timesteps % self.eval_every == 0:
            self.record_video()
        return True

    def record_video(self) -> None:
        print("recording video")
        obs, info = self.eval_env.reset()
        video = [self.eval_env.render().transpose(2, 0, 1)]
        for _ in range(1000):
            action = self.model.predict(obs, deterministic=True)[0]
            obs, _, terminated, truncated, _ = self.eval_env.step(action)
            pixels = self.eval_env.render()
            if pixels is not None:
                video.append(pixels.transpose(2, 0, 1))
            if terminated or truncated:
                break

        if video:
            video = np.stack(video)
            wandb.log({"results/video": wandb.Video(video, fps=10, format="mp4")}, step=self.model.num_timesteps)
        else:
            log.warning("No video recorded")


class LogCallback(BaseCallback):
    """
    Custom callback for plotting additional values in tensorboard.
    """

    def __init__(self, verbose=0, info_keywords=(), args=None):
        super().__init__(verbose)
        self.aux_rewards = {}
        self.aux_returns = {}
        for key in info_keywords:
            self.aux_rewards[key] = np.zeros(args.num_envs)
            self.aux_returns[key] = deque(maxlen=100)

    def _on_step(self) -> bool:
        infos = self.locals["infos"]
        for idx in range(len(infos)):
            for key in self.aux_rewards.keys():
                self.aux_rewards[key][idx] += infos[idx][key]

            if self.locals["dones"][idx]:
                for key in self.aux_rewards.keys():
                    self.aux_returns[key].append(self.aux_rewards[key][idx])
                    self.aux_rewards[key][idx] = 0
        return True

    def _on_rollout_end(self) -> None:
        global_step = self.model.num_timesteps
        for key in self.aux_returns.keys():
            self.logger.record(f"aux_returns_{key}/mean", np.mean(self.aux_returns[key]), global_step)


def main():
    args = parse_args()

    if args.use_wandb:
        run = wandb.init(
            project=args.wandb_project,
            entity=args.wandb_entity,
            config=vars(args),
            sync_tensorboard=True,
            monitor_gym=True,
            save_code=True,
            name=f"SB3-{args.sim}-{args.robot}-{args.task}-{time.strftime('%Y_%m_%d_%H_%M_%S')}",
        )
    else:
        from collections import namedtuple

        Run = namedtuple("Run", ["id"])
        run = Run(id=int(time.time()))

    # Create scenario config
    task = get_task(args.task)
    robot = get_robot(args.robot)
    scenario = ScenarioMetaCfg(task=task, robot=robot, try_add_table=args.add_table, sim=args.sim)

    env = make_vec_env(
        lambda: MetaSimGymWrapper(
            scenario=scenario,
            sim_type=SimType(args.sim),
            num_envs=1,  # Each instance is a single environment
            headless=args.train_or_eval == "train",
        ),
        n_envs=args.num_envs,
    )

    # Initialize PPO algorithm
    model = PPO(
        policy="MlpPolicy",
        env=env,
        learning_rate=args.learning_rate,
        n_steps=args.n_steps,
        batch_size=args.batch_size,
        n_epochs=args.n_epochs,
        verbose=1,
        tensorboard_log=f"./ppo_logs/{run.id}",
        device="cpu",
    )

    # Setup wandb callback with additional monitoring options
    wandb_callback = (
        WandbCallback(
            model_save_freq=100000,
            model_save_path=f"models/{run.id}",
            verbose=2,
            gradient_save_freq=100000,  # Added to track gradients
        )
        if args.use_wandb
        else None
    )

    if args.train_or_eval == "train":
        # Train the agent with additional callbacks
        log.info("Starting training...")
        model.learn(
            total_timesteps=args.total_timesteps,
            log_interval=1,
            callback=[
                EvalCallback(scenario=scenario, sim_type=SimType(args.sim)),
                EpisodeLogCallback(),
                LogCallback(args=args),
            ]
            + ([wandb_callback] if args.use_wandb else []),
            progress_bar=True,
        )

        # Save the trained model
        model_path = f"models/ppo_model_{args.robot}_{args.sim}_{run.id}"
        model.save(model_path)
        log.info(f"Model saved to {model_path}")
    elif args.train_or_eval == "eval":
        # Load the trained model
        model.load(args.model_path)

        # Evaluate the agent
        log.info("Starting evaluation...")
        obs, info = env.reset()
        rewards = []
        for _ in range(1000):
            action, _ = model.predict(obs, deterministic=True)
            obs, reward, terminated, truncated, info = env.step(action)
            rewards.append(reward)
            env.render()
            if terminated or truncated:
                obs, info = env.reset()
                import matplotlib.pyplot as plt

                plt.plot(rewards)
                plt.savefig("rewards.png")
                plt.clf()
                rewards = []

    elif args.train_or_eval == "test":
        # Test the joints
        log.info("Starting evaluation...")
        obs, info = env.reset()
        rewards = []

        step = 0
        target_joint = 0
        while True:
            step += 1
            action = np.array(
                [0] * 19,
                dtype=np.float32,
            )
            action[target_joint] = np.sin(step / 10)
            # action = env.normalize_action(np.array([0,0,-0.4,0.8,-0.4,0,0,-0.4,0.8,-0.4,0,0,0,0,0,0,0,0,0]))
            # action = env.normalize_action(obs[7:26])
            obs, reward, terminated, truncated, info = env.step(action)
            env.render()

    # Close environment and wandb
    env.close()
    wandb.finish()


if __name__ == "__main__":
    main()



import math

from metasim.cfg.checkers import AndOp, DetectedChecker, Relative2DSphereDetector, RelativeBboxDetector
from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass

from ..base_task_metacfg import BaseTaskMetaCfg


@configclass
class SquareD1MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.ROBOSUITE
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 200
    traj_filepath = "data_isaaclab/source_data/robosuite/square_d1/square_d1_v2.pkl"
    objects = [
        RigidObjMetaCfg(
            name="peg1",
            filepath="data_isaaclab/assets/robosuite/square_d0/pegs_arena/peg1.usd",
            physics=PhysicStateType.XFORM,
        ),
        PrimitiveCubeMetaCfg(
            name="table",
            # texture = 'data_isaaclab/robosuite/textures/ceramic.png',
            size=[0.8, 0.8, 0.05],
            physics=PhysicStateType.GEOM,
            color=[1.0, 1.0, 1.0],
        ),
        RigidObjMetaCfg(
            name="SquareNut",
            filepath="data_isaaclab/assets/robosuite/square_d0/square_nut/square_nut.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    checker = AndOp([
        DetectedChecker(
            obj_name="SquareNut",
            detector=RelativeBboxDetector(
                base_obj_name="peg1",
                relative_pos=[0.0, 0.0, 0.05],
                relative_quat=[1.0, 0.0, 0.0, 0.0],
                checker_lower=[-math.inf, -math.inf, -math.inf],
                checker_upper=[math.inf, math.inf, 0.0],
                debug_vis=True,
            ),
        ),
        DetectedChecker(
            obj_name="SquareNut",
            detector=Relative2DSphereDetector(
                base_obj_name="peg1",
                relative_pos=[0.0, 0.0, 0.05],
                aixs=[0, 1],
                radius=0.02,
            ),
        ),
    ])



from metasim.cfg.objects import RigidObjMetaCfg
from metasim.constants import PhysicStateType
from metasim.utils import configclass

from .rlbench_task_metacfg import RLBenchTaskMetaCfg


@configclass
class PhoneOnBaseMetaCfg(RLBenchTaskMetaCfg):
    episode_length = 200
    traj_filepath = "roboverse_data/trajs/rlbench/phone_on_base/v2"
    objects = [
        RigidObjMetaCfg(
            name="phone_visual",
            filepath="roboverse_data/assets/rlbench/phone_on_base/phone_visual/usd/phone_visual.usd",
            physics=PhysicStateType.XFORM,
        ),
        RigidObjMetaCfg(
            name="phone_case_visual",
            filepath="roboverse_data/assets/rlbench/phone_on_base/phone_case_visual/usd/phone_case_visual.usd",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    # TODO: add checker



from __future__ import annotations

import random
from dataclasses import dataclass

import gymnasium as gym
import numpy as np
import rootutils
import torch
import tyro
from gymnasium import spaces
from gymnasium.vector import VectorEnv
from loguru import logger as log
from rich.logging import RichHandler
from stable_baselines3 import PPO
from stable_baselines3.common.vec_env import VecEnv

rootutils.setup_root(__file__, pythonpath=True)
log.configure(handlers=[{"sink": RichHandler(), "format": "{message}"}])

from metasim.cfg.scenario import ScenarioMetaCfg
from metasim.constants import SimType
from metasim.sim.base import BaseSimHandler
from metasim.utils.demo_util import get_traj
from metasim.utils.setup_util import get_sim_env_class


@dataclass
class Args(ScenarioMetaCfg):
    task: str = "reach_origin"
    robot: str = "franka"
    num_envs: int = 16


args = tyro.cli(Args)


class MetaSimVecEnv(VectorEnv):
    """Vectorized environment for MetaSim that supports parallel RL training"""

    def __init__(
        self,
        scenario: ScenarioMetaCfg | None = None,
        sim_type: SimType = SimType.ISAACLAB,
        task_name: str | None = None,
        num_envs: int | None = 4,
    ):
        if scenario is None:
            scenario = ScenarioMetaCfg(task="pick_cube", robot="franka")
            scenario.task = task_name
            scenario.num_envs = num_envs
            scenario = ScenarioMetaCfg(**vars(scenario))
        self.num_envs = scenario.num_envs
        env_class = get_sim_env_class(sim_type)
        env = env_class(scenario, scenario.num_envs)
        self.handler: BaseSimHandler = env.handler
        self.render_mode = None  # XXX
        self.scenario = scenario

        # Get candidate states
        self.candidate_init_states, _, _ = get_traj(scenario.task, scenario.robot)

        # XXX: is the inf space ok?
        super().__init__(self.num_envs, spaces.Box(-np.inf, np.inf), spaces.Box(-np.inf, np.inf))

    ############################################################
    ## Gym-like interface
    ############################################################
    def reset(self, seed: int | None = None):
        init_states = self.unwrapped._get_default_states(seed)
        self.handler.set_states(init_states)
        self.handler.reset()
        return self.unwrapped._get_obs(), {}

    def step(self, actions: list[dict]):
        _, _, success, timeout, _ = self.handler.step(actions)
        obs = self.unwrapped._get_obs()
        rewards = self.unwrapped._calculate_rewards()
        return obs, rewards, success, timeout, {}

    def render(self):
        return self.handler.render()

    def close(self):
        self.handler.close()

    ############################################################
    ## Helper methods
    ############################################################
    def _get_obs(self):
        ## TODO: put this function into task definition?
        ## TODO: use torch instead of numpy
        """Get current observations for all environments"""
        states = self.handler.get_states()
        joint_pos = torch.tensor([
            [state[self.handler.robot.name]["dof_pos"][j] for j in self.handler.robot.joint_limits.keys()]
            for state in states
        ])

        # Get end effector positions (assuming 'ee' is the end effector subpath)
        ee_pos = torch.stack([state["metasim_body_panda_hand"]["pos"] for state in states])

        return torch.cat([joint_pos, ee_pos], dim=1)

    def _calculate_rewards(self):
        """Calculate rewards based on distance to origin"""
        states = self.handler.get_states()
        tot_reward = torch.zeros(self.num_envs)
        for reward_fn, weight in zip(self.scenario.task.reward_functions, self.scenario.task.reward_weights):
            tot_reward += weight * reward_fn(states)
        return tot_reward

    def _get_default_states(self, seed: int | None = None):
        """Generate default reset states"""
        ## TODO: use non-reqeatable random choice when there is enough candidate states?
        return random.Random(seed).choices(self.candidate_init_states, k=self.num_envs)


class StableBaseline3VecEnv(VecEnv):
    def __init__(self, env: MetaSimVecEnv):
        joint_limits = env.scenario.robot.joint_limits

        # TODO: customize action space?
        self.action_space = spaces.Box(
            low=np.array([lim[0] for lim in joint_limits.values()]),
            high=np.array([lim[1] for lim in joint_limits.values()]),
            dtype=np.float32,
        )

        # TODO: customize observation space?
        # Observation space: joint positions + end effector position
        self.observation_space = spaces.Box(
            low=-np.inf,
            high=np.inf,
            shape=(len(joint_limits) + 3,),  # joints + XYZ
            dtype=np.float32,
        )

        self.env = env
        self.render_mode = None  # XXX
        super().__init__(self.env.num_envs, self.observation_space, self.action_space)

    ############################################################
    ## Gym-like interface
    ############################################################
    def reset(self):
        obs, _ = self.env.reset()
        return obs.numpy()

    def step_async(self, actions: np.ndarray) -> None:
        self.action_dicts = [
            {"dof_pos_target": dict(zip(self.env.scenario.robot.joint_limits.keys(), action))} for action in actions
        ]

    def step_wait(self):
        obs, rewards, success, timeout, _ = self.env.step(self.action_dicts)

        dones = success | timeout
        if dones.any():
            init_states = self.env.unwrapped._get_default_states()
            self.env.handler.set_states(init_states, env_ids=np.nonzero(dones)[0].tolist())
            self.env.handler.reset(dones.nonzero().squeeze(-1).tolist())

        extra = [{} for _ in range(self.num_envs)]
        for env_id in range(self.num_envs):
            if dones[env_id]:
                extra[env_id]["terminal_observation"] = obs[env_id]
            extra[env_id]["TimeLimit.truncated"] = timeout[env_id] and not success[env_id]

        obs = self.env.unwrapped._get_obs()

        return obs.numpy(), rewards.numpy(), dones.numpy(), extra

    def render(self):
        return self.env.render()

    def close(self):
        self.env.close()

    ############################################################
    ## Abstract methods
    ############################################################
    def get_images(self):
        raise NotImplementedError

    def get_attr(self, attr_name, indices=None):
        if indices is None:
            indices = list(range(self.num_envs))
        return [getattr(self.env.handler, attr_name)] * len(indices)

    def set_attr(self, attr_name: str, value, indices=None) -> None:
        raise NotImplementedError

    def env_method(self, method_name: str, *method_args, indices=None, **method_kwargs):
        raise NotImplementedError

    def env_is_wrapped(self, wrapper_class, indices=None):
        raise NotImplementedError


def train_ppo():
    ## Choice 1: use scenario config to initialize the environment
    # scenario = ScenarioMetaCfg(**vars(args))
    # metasim_env = MetaSimVecEnv(scenario)

    ## Choice 2: use gym.make to initialize the environment
    metasim_env = gym.make("reach_origin", num_envs=args.num_envs)
    env = StableBaseline3VecEnv(metasim_env)

    # PPO configuration
    model = PPO(
        "MlpPolicy",
        env,
        verbose=1,
        learning_rate=3e-4,
        n_steps=128,
        batch_size=64,
        n_epochs=10,
        gamma=0.99,
        gae_lambda=0.95,
        clip_range=0.2,
        device="cuda" if torch.cuda.is_available() else "cpu",
    )

    # Start training
    model.learn(total_timesteps=1_000_000)
    model.save("ppo_reach")


if __name__ == "__main__":
    train_ppo()



from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass


@configclass
class OpensdorRotCapLeftBottomRightBottle7MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/022_windex_bottle_google_16k/022_windex_bottle_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/022_cap_left_bottom_right/20240826-212311_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle8MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/022_windex_bottle_google_16k/022_windex_bottle_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/022_cap_left_bottom_right/20240826-224706_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle9MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/022_windex_bottle_google_16k/022_windex_bottle_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/022_cap_left_bottom_right/20240826-213221_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle184MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/a7785ac7ce7f4ba79df661db33637ba9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/a7785ac7ce7f4ba79df661db33637ba9_cap_left_bottom_right/20240826-214206_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle185MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/a7785ac7ce7f4ba79df661db33637ba9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/a7785ac7ce7f4ba79df661db33637ba9_cap_left_bottom_right/20240826-224518_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle186MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/a7785ac7ce7f4ba79df661db33637ba9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/a7785ac7ce7f4ba79df661db33637ba9_cap_left_bottom_right/20240826-224045_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle187MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/a7785ac7ce7f4ba79df661db33637ba9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/a7785ac7ce7f4ba79df661db33637ba9_cap_left_bottom_right/20240826-225839_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle188MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/a7785ac7ce7f4ba79df661db33637ba9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/a7785ac7ce7f4ba79df661db33637ba9_cap_left_bottom_right/20240826-211513_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle189MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/a7785ac7ce7f4ba79df661db33637ba9/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/a7785ac7ce7f4ba79df661db33637ba9_cap_left_bottom_right/20240826-225928_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle623MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/006_mustard_bottle_google_16k/006_mustard_bottle_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/006_cap_left_bottom_right/20240826-224634_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle624MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/006_mustard_bottle_google_16k/006_mustard_bottle_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/006_cap_left_bottom_right/20240826-223740_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle625MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/006_mustard_bottle_google_16k/006_mustard_bottle_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/006_cap_left_bottom_right/20240826-223040_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle626MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/006_mustard_bottle_google_16k/006_mustard_bottle_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/006_cap_left_bottom_right/20240826-225957_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle627MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/006_mustard_bottle_google_16k/006_mustard_bottle_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/006_cap_left_bottom_right/20240826-215005_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle778MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/8c0f8088a5d546c886d1b07e3f4eafae/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/8c0f8088a5d546c886d1b07e3f4eafae_cap_left_bottom_right/20240826-222522_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle779MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/8c0f8088a5d546c886d1b07e3f4eafae/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/8c0f8088a5d546c886d1b07e3f4eafae_cap_left_bottom_right/20240826-213928_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle780MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/8c0f8088a5d546c886d1b07e3f4eafae/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/8c0f8088a5d546c886d1b07e3f4eafae_cap_left_bottom_right/20240826-224111_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle781MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/8c0f8088a5d546c886d1b07e3f4eafae/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/8c0f8088a5d546c886d1b07e3f4eafae_cap_left_bottom_right/20240826-223010_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle782MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/8c0f8088a5d546c886d1b07e3f4eafae/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/8c0f8088a5d546c886d1b07e3f4eafae_cap_left_bottom_right/20240826-223656_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle783MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/8c0f8088a5d546c886d1b07e3f4eafae/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/8c0f8088a5d546c886d1b07e3f4eafae_cap_left_bottom_right/20240826-212213_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle869MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/021_bleach_cleanser_google_16k/021_bleach_cleanser_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/021_cap_left_bottom_right/20240826-230315_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle870MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/021_bleach_cleanser_google_16k/021_bleach_cleanser_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/021_cap_left_bottom_right/20240826-224836_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle871MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/021_bleach_cleanser_google_16k/021_bleach_cleanser_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/021_cap_left_bottom_right/20240826-224920_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle872MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path="data_isaaclab/assets/open6dor/ycb_16k_backup/021_bleach_cleanser_google_16k/021_bleach_cleanser_google_16k.urdf",
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/021_cap_left_bottom_right/20240826-212914_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1126MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/6add0a1fa81942acbb24504b130661c1/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/6add0a1fa81942acbb24504b130661c1_cap_left_bottom_right/20240826-223729_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1127MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/6add0a1fa81942acbb24504b130661c1/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/6add0a1fa81942acbb24504b130661c1_cap_left_bottom_right/20240826-221136_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1174MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/794b730ae452424bb3a9ce3c6caaff7a/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/794b730ae452424bb3a9ce3c6caaff7a_cap_left_bottom_right/20240826-214902_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1175MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/794b730ae452424bb3a9ce3c6caaff7a/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/794b730ae452424bb3a9ce3c6caaff7a_cap_left_bottom_right/20240826-212108_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1176MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/794b730ae452424bb3a9ce3c6caaff7a/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/794b730ae452424bb3a9ce3c6caaff7a_cap_left_bottom_right/20240826-225226_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1177MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/794b730ae452424bb3a9ce3c6caaff7a/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/794b730ae452424bb3a9ce3c6caaff7a_cap_left_bottom_right/20240826-223209_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1259MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d17b8f44d1544364b8ad9fe2554ddfdf/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d17b8f44d1544364b8ad9fe2554ddfdf_cap_left_bottom_right/20240826-212557_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1260MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d17b8f44d1544364b8ad9fe2554ddfdf/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d17b8f44d1544364b8ad9fe2554ddfdf_cap_left_bottom_right/20240826-220543_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1261MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d17b8f44d1544364b8ad9fe2554ddfdf/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d17b8f44d1544364b8ad9fe2554ddfdf_cap_left_bottom_right/20240826-215556_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1262MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d17b8f44d1544364b8ad9fe2554ddfdf/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d17b8f44d1544364b8ad9fe2554ddfdf_cap_left_bottom_right/20240826-212426_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1263MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d17b8f44d1544364b8ad9fe2554ddfdf/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d17b8f44d1544364b8ad9fe2554ddfdf_cap_left_bottom_right/20240826-222918_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotCapLeftBottomRightBottle1264MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="bottle",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d17b8f44d1544364b8ad9fe2554ddfdf/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d17b8f44d1544364b8ad9fe2554ddfdf_cap_left_bottom_right/20240826-221511_no_interaction/trajectory-unified_wo_traj_v2.pkl"



from metasim.cfg.objects import PrimitiveCubeMetaCfg, RigidObjMetaCfg
from metasim.cfg.tasks.base_task_metacfg import BaseTaskMetaCfg
from metasim.constants import BenchmarkType, PhysicStateType, TaskType
from metasim.utils import configclass


@configclass
class OpensdorRotWatchUprightWatch71MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d6e38c73f8f2402aac890e8ebb115302/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d6e38c73f8f2402aac890e8ebb115302_watch_upright/20240826-213508_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch72MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d6e38c73f8f2402aac890e8ebb115302/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d6e38c73f8f2402aac890e8ebb115302_watch_upright/20240826-212139_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch73MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d6e38c73f8f2402aac890e8ebb115302/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d6e38c73f8f2402aac890e8ebb115302_watch_upright/20240826-212259_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch74MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d6e38c73f8f2402aac890e8ebb115302/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d6e38c73f8f2402aac890e8ebb115302_watch_upright/20240826-211531_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch75MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d6e38c73f8f2402aac890e8ebb115302/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d6e38c73f8f2402aac890e8ebb115302_watch_upright/20240826-213219_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch76MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/d6e38c73f8f2402aac890e8ebb115302/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/d6e38c73f8f2402aac890e8ebb115302_watch_upright/20240826-212821_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch149MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/fdb8227a88b2448d8c64fa82ffee3f58/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/fdb8227a88b2448d8c64fa82ffee3f58_watch_upright/20240826-212915_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch150MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/fdb8227a88b2448d8c64fa82ffee3f58/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/fdb8227a88b2448d8c64fa82ffee3f58_watch_upright/20240826-213759_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch151MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/fdb8227a88b2448d8c64fa82ffee3f58/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/fdb8227a88b2448d8c64fa82ffee3f58_watch_upright/20240826-214854_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch152MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/fdb8227a88b2448d8c64fa82ffee3f58/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/fdb8227a88b2448d8c64fa82ffee3f58_watch_upright/20240826-211453_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch153MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/fdb8227a88b2448d8c64fa82ffee3f58/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/fdb8227a88b2448d8c64fa82ffee3f58_watch_upright/20240826-212156_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch154MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/fdb8227a88b2448d8c64fa82ffee3f58/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/fdb8227a88b2448d8c64fa82ffee3f58_watch_upright/20240826-212743_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch568MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/243b381dcdc34316a7e78a533572d273/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/243b381dcdc34316a7e78a533572d273_watch_upright/20240826-212236_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch569MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/243b381dcdc34316a7e78a533572d273/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/243b381dcdc34316a7e78a533572d273_watch_upright/20240826-212120_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch570MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/243b381dcdc34316a7e78a533572d273/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/243b381dcdc34316a7e78a533572d273_watch_upright/20240826-215225_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch571MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/243b381dcdc34316a7e78a533572d273/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/243b381dcdc34316a7e78a533572d273_watch_upright/20240826-212624_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch572MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/243b381dcdc34316a7e78a533572d273/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/243b381dcdc34316a7e78a533572d273_watch_upright/20240826-215959_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch573MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/243b381dcdc34316a7e78a533572d273/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/243b381dcdc34316a7e78a533572d273_watch_upright/20240826-220308_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch574MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/243b381dcdc34316a7e78a533572d273/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/243b381dcdc34316a7e78a533572d273_watch_upright/20240826-215458_no_interaction/trajectory-unified_wo_traj_v2.pkl"


@configclass
class OpensdorRotWatchUprightWatch575MetaCfg(BaseTaskMetaCfg):
    source_benchmark = BenchmarkType.OPEN6DOR
    task_type = TaskType.TABLETOP_MANIPULATION
    episode_length = 600
    objects = [
        PrimitiveCubeMetaCfg(
            name="table",
            mass=1,
            color=[255, 255, 255],
            size=[0.6, 0.8, 0.3],
        ),
        RigidObjMetaCfg(
            name="watch",
            urdf_path=(
                "data_isaaclab/assets/open6dor/objaverse_rescale/243b381dcdc34316a7e78a533572d273/material_2.urdf"
            ),
            physics=PhysicStateType.RIGIDBODY,
        ),
    ]
    traj_filepath = "data_isaaclab/source_data/open6dor/task_refine_rot_only/rot_ins/243b381dcdc34316a7e78a533572d273_watch_upright/20240826-211447_no_interaction/trajectory-unified_wo_traj_v2.pkl"



# Copyright (c) 2022-2024, The Isaac Lab Project Developers.
# All rights reserved.
#
# SPDX-License-Identifier: BSD-3-Clause

"""Sub-module for utilities for working with dictionaries."""

from __future__ import annotations

from collections.abc import Iterable, Mapping
from dataclasses import is_dataclass
from typing import Any

from .string_util import callable_to_string, string_to_callable, string_to_dataclass_instance

"""
Dictionary <-> Class operations.
"""


def class_to_dict(obj: object) -> dict[str, Any]:
    """Convert an object into dictionary recursively.

    Note:
        Ignores all names starting with "__" (i.e. built-in methods).

    Args:
        obj: An instance of a class to convert.

    Raises:
        ValueError: When input argument is not an object.

    Returns:
        Converted dictionary mapping.
    """
    # check that input data is class instance
    if not hasattr(obj, "__class__"):
        raise ValueError(f"Expected a class instance. Received: {type(obj)}.")
    # convert object to dictionary
    if isinstance(obj, dict):
        obj_dict = obj
    elif hasattr(obj, "__dict__"):
        obj_dict = obj.__dict__
    else:
        return obj

    # convert to dictionary
    data = dict()
    for key, value in obj_dict.items():
        # disregard builtin attributes
        if key.startswith("__"):
            continue
        # check if attribute is callable -- function
        if callable(value):
            data[key] = callable_to_string(value)
        # check if attribute is a dictionary
        elif hasattr(value, "__dict__") or isinstance(value, dict):
            data[key] = class_to_dict(value)
        elif isinstance(value, (list, tuple)):
            data[key] = type(value)([class_to_dict(v) for v in value])
        else:
            data[key] = value
    return data


def update_class_from_dict(obj, data: dict[str, Any], _ns: str = "") -> None:
    """Reads a dictionary and sets object variables recursively.

    This function performs in-place update of the class member attributes.

    Args:
        obj: An instance of a class to update.
        data: Input dictionary to update from.
        _ns: Namespace of the current object. This is useful for nested configuration
            classes or dictionaries. Defaults to "".

    Raises:
        TypeError: When input is not a dictionary.
        ValueError: When dictionary has a value that does not match default config type.
        KeyError: When dictionary has a key that does not exist in the default config type.
    """
    for key, value in data.items():
        # key_ns is the full namespace of the key
        key_ns = _ns + "/" + key
        # check if key is present in the object
        if hasattr(obj, key) or isinstance(obj, dict):
            obj_mem = obj[key] if isinstance(obj, dict) else getattr(obj, key)
            if isinstance(value, Mapping):
                # recursively call if it is a dictionary
                update_class_from_dict(obj_mem, value, _ns=key_ns)
                continue
            if isinstance(value, Iterable) and not isinstance(value, str):
                # check length of value to be safe
                if len(obj_mem) != len(value) and obj_mem is not None:
                    raise ValueError(
                        f"[Config]: Incorrect length under namespace: {key_ns}."
                        f" Expected: {len(obj_mem)}, Received: {len(value)}."
                    )
                if isinstance(obj_mem, tuple):
                    value = tuple(value)
                else:
                    set_obj = True
                    # recursively call if iterable contains dictionaries
                    for i in range(len(obj_mem)):
                        if isinstance(value[i], dict):
                            update_class_from_dict(obj_mem[i], value[i], _ns=key_ns)
                            set_obj = False
                    # do not set value to obj, otherwise it overwrites the cfg class with the dict
                    if not set_obj:
                        continue
            elif callable(obj_mem):
                # update function name
                value = string_to_callable(value)
            elif is_dataclass(obj_mem):
                value = string_to_dataclass_instance(value)
            elif isinstance(value, type(obj_mem)) or value is None:
                pass
            else:
                raise ValueError(
                    f"[Config]: Incorrect type under namespace: {key_ns}."
                    f" Expected: {type(obj_mem)}, Received: {type(value)}."
                )
            # set value
            if isinstance(obj, dict):
                obj[key] = value
            else:
                setattr(obj, key, value)
        else:
            raise KeyError(f"[Config]: Key not found under namespace: {key_ns}.")



from __future__ import annotations

from dataclasses import MISSING

from metasim.cfg.objects import ArticulationObjMetaCfg
from metasim.utils import configclass


@configclass
class BaseActuatorMetaCfg:
    velocity_limit: float | None = None  # TODO: None means use the default value (USD joint prim value) or no limit?
    is_ee: bool = False
    damping: float = 40.0
    stiffness: float = 400.0


@configclass
class BaseRobotMetaCfg(ArticulationObjMetaCfg):
    # Articulation
    num_joints: int = MISSING
    actuators: dict[str, BaseActuatorMetaCfg] = {}
    ee_prim_path: str | None = None
    fix_base_link: bool = True
    joint_limits: dict[str, tuple[float, float]] = {}
    default_joint_positions: dict[str, float] = {}
    default_position: tuple[float, float, float] = (0.0, 0.0, 0.0)
    """
    Joint limits in the format of `{joint_name: (lower_limit, upper_limit)}`.
    Note that different simulators may have different order of joints, so you should not use the order in this dict!
    """
    ee_binary_action: bool = False
    gripper_release_q: list[float] = MISSING
    gripper_actuate_q: list[float] = MISSING

    # cuRobo Configs
    curobo_ref_cfg_name: str = MISSING
    curobo_tcp_rel_pos: tuple[float, float, float] = MISSING
    curobo_tcp_rel_rot: tuple[float, float, float] = MISSING

    # Simulation
    enabled_gravity: bool = True
    enabled_self_collisions: bool = True
    isaacgym_flip_visual_attachments: bool = True



from __future__ import annotations

from metasim.utils import configclass

from .base_robot_metacfg import BaseActuatorMetaCfg, BaseRobotMetaCfg


@configclass
class SawyerMujocoMetaCfg(BaseRobotMetaCfg):
    name: str = "sawyer"
    num_joints: int = 9
    filepath: str = "roboverse_data/robots/sawyer/usd/sawyer_mujoco_v1.usd"
    mjcf_path: str = "roboverse_data/robots/sawyer/mjcf/sawyer.xml"
    freejoint: bool = False
    enabled_gravity: bool = False
    enabled_self_collisions: bool = False
    actuators: dict[str, BaseActuatorMetaCfg] = {
        "right_j0": BaseActuatorMetaCfg(),
        "right_j1": BaseActuatorMetaCfg(),
        "right_j2": BaseActuatorMetaCfg(),
        "right_j3": BaseActuatorMetaCfg(),
        "right_j4": BaseActuatorMetaCfg(),
        "right_j5": BaseActuatorMetaCfg(),
        "right_j6": BaseActuatorMetaCfg(),
        "r_close": BaseActuatorMetaCfg(is_ee=True),
        "l_close": BaseActuatorMetaCfg(is_ee=True),
    }
    joint_limits: dict[str, tuple[float, float]] = {
        "right_j0": (-3.0503, 3.0503),
        "right_j1": (-3.8, -0.5),
        "right_j2": (-3.0426, 3.0426),
        "right_j3": (-3.0439, 3.0439),
        "right_j4": (-2.9761, 2.9761),
        "right_j5": (-2.9761, 2.9761),
        "right_j6": (-4.7124, 4.7124),
        "r_close": (0.0, 0.04),
        "l_close": (-0.03, 0),
    }
    # ee_prim_path: str = "sawyer_right_hand"
    # gripper_release_q = [0.020833, -0.020833]
    # gripper_actuate_q = [0.0, 0.0]

    # curobo_ref_cfg_name: str = "sawyer.yml"
    # curobo_tcp_rel_pos: tuple[float, float, float] = [0.0, 0.0, 0.105]
    # curobo_tcp_rel_rot: tuple[float, float, float] = [0.0, 0.0, 0.0]



from metasim.cfg.checkers import StandChecker
from metasim.utils import configclass

from .base_locomotion_env import BaseLocomotionReward, HumanoidTaskMetaCfg


class StandReward(BaseLocomotionReward):
    _move_speed = 0
    success_bar = 800


@configclass
class StandMetaCfg(HumanoidTaskMetaCfg):
    episode_length = 1000
    # traj_filepath = "roboverse_data/trajs/humanoidbench/stand/v2/h1_v2.pkl"
    traj_filepath = "roboverse_data/trajs/humanoidbench/stand/v2/initial_state_v2.json"
    checker = StandChecker()
    reward_weights = [1.0]
    reward_functions = [StandReward]
